/***************************************************************
 * Name:      editordialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <QPrinter>
#include <QPrintDialog>
#include "editordialog.h"
#include "previewdialog.h"
#include "findreplacedialog.h"
#include "dxutils.h"

TemplateDialog::TemplateDialog(QWidget *parent)
    : QDialog(parent)
{
    QGridLayout *grid = new QGridLayout;
    grid->addWidget(new QLabel(QString::fromUtf8("Šablona")),0,0);
    m_template = new QComboBox;
    grid->addWidget(m_template,0,1,1,2);
    m_template->addItem(QString::fromUtf8("Faktura vydaná neplátce"), "favydananeplatce.html");
    m_template->addItem(QString::fromUtf8("Faktura vydaná plátce"), "favydanaplatce.html");
    m_template->addItem(QString::fromUtf8("Zálohová faktura vydaná neplátce"), "zalohavydananeplatce.html");
    m_template->addItem(QString::fromUtf8("Zálohová faktura vydaná plátce"), "zalohavydanaplatce.html");
    m_template->addItem(QString::fromUtf8("Faktura přijatá neplátce"), "faprijataneplatce.html");
    m_template->addItem(QString::fromUtf8("Faktura přijatá plátce"), "faprijataplatce.html");
    m_template->addItem(QString::fromUtf8("Zálohová faktura přijatá neplátce"), "zalohaprijataneplatce.html");
    m_template->addItem(QString::fromUtf8("Zálohová faktura přijatá plátce"), "zalohaprijataplatce.html");
    QDialogButtonBox *buttons = new QDialogButtonBox(QDialogButtonBox::Ok|QDialogButtonBox::Cancel);
    connect(buttons, SIGNAL(rejected()), this, SLOT(reject()));
    connect(buttons, SIGNAL(accepted()), this, SLOT(accept()));
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(grid);
    mainLayout->addWidget(buttons);
    setLayout(mainLayout);
    setWindowTitle(QString::fromUtf8("Vyberte typ šablony"));
}

QString TemplateDialog::templateName() const
{
    return m_template->itemData(m_template->currentIndex()).toString();
}

TemplateHighlighter::TemplateHighlighter(QTextDocument *parent)
    : QSyntaxHighlighter(parent)
{
    m_keywords <<"{firma-nazev}"<<"{firma-ulice}"<<"{firma-registrace}"<<"{firma-logo}"<<"{firma-razitko}"<<
            "{firma-psc}"<<"{firma-misto}"<<"{firma-zeme}"<<"{firma-ic}"<<"{firma-dic}"<<
            "{firma-telefon}"<<"{firma-email}"<<"{firma-www}"
            << "{faktura-cislo}"<<"{faktura-celkem}"<<"{faktura-uhrada}"<<"{faktura-banka}"<<
            "{faktura-vystaveni}"<<"{faktura-splatnost}"<<"{faktura-duzp}"<<
            "{faktura-ks}"<<"{faktura-vs}"<<"{faktura-mena}"
            << "{adresar-nazev}"<<"{adresar-ulice}"<<"{adresar-psc}"<<"{adresar-misto}"<<
            "{adresar-zeme}"<<"{adresar-ic}"<<"{adresar-dic}"
            << "{polozka-popis}"<<"{polozka-pocet}"<<"{polozka-cenamj}"<<
            "{polozka-sazba}"<<"{polozka-dph}"<<"{polozka-celkem}"
            << "{dph-sazba}"<<"{dph-zaklad}"<<"{dph-dph}";
    m_keywordFormat.setForeground(Qt::darkBlue);
    m_keywordFormat.setFontWeight(QFont::Bold);
    m_tagFormat.setFontItalic(true);
    m_tagFormat.setForeground(QColor("orange"));
}

void TemplateHighlighter::highlightBlock(const QString &text)
{
    int start = 0;
    int i = 0;
    int length = text.length();
    int count = 0;
    QChar ch;
    while(i < length)
    {
        ch = (i < length) ? text.at(i) : QChar();
        if(ch == '<')
        {
            start = i;
            count = 0;
            while(i < length)
            {
                ch = (i < length) ? text.at(i) : QChar();
                if(ch == '>')
                {
                    count = i-start+1;
                    break;
                }
                else
                {
                    ++i;
                }
            }
            setFormat(start, count, m_tagFormat);
        }
        else if(ch == '{')
        {
            start = i;
            count = 0;
            while(i < length)
            {
                ch = (i < length) ? text.at(i) : QChar();
                if(ch == '}')
                {
                    QString word = text.mid(start, i-start+1);
                    if(m_keywords.contains(word)) count = i-start+1;
                    break;
                }
                else
                {
                    ++i;
                }
            }
            setFormat(start, count, m_keywordFormat);
        }
        else
        {
            ++i;
        }
    }
}

EditorDialog::EditorDialog(QWidget *parent) :
    QMainWindow(parent)
{
    QFont font;
    font.setFamily("Courier");
    font.setFixedPitch(true);
    font.setPointSize(10);
    m_edit = new QTextEdit;
    m_edit->setReadOnly(true);
    m_edit->setFont(font);
    m_highlighter = new TemplateHighlighter(m_edit->document());
    connect(m_edit->document(), SIGNAL(contentsChanged()), this, SLOT(slotDocumentChanged()));
    m_edit->document()->setModified(false);
    setCentralWidget(m_edit);
    createActions();
    createMenu();
    createToolbar();
    resize(800, 600);
    setWindowTitle(QString::fromUtf8("Editor šablon"));
}

void EditorDialog::createActions()
{
    m_open = new QAction(QIcon(":/icons/open.png"), QString::fromUtf8("Otevřít"), this);
    m_open->setShortcut(QKeySequence::Open);
    connect(m_open, SIGNAL(triggered()), this, SLOT(slotCmdOpen()));
    m_save = new QAction(QIcon(":/icons/save.png"), QString::fromUtf8("Uložit"), this);
    m_save->setShortcut(QKeySequence::Save);
    connect(m_save, SIGNAL(triggered()), this, SLOT(slotCmdSave()));
    m_print = new QAction(QIcon(":/icons/print.png"), "Vytisknout", this);
    m_print->setShortcut(QKeySequence::Print);
    connect(m_print, SIGNAL(triggered()), this, SLOT(slotCmdPrint()));
    m_print->setEnabled(false);
    m_preview = new QAction(QIcon(":/icons/preview.png"), QString::fromUtf8("Náhled šablony"), this);
    m_preview->setEnabled(false);
    connect(m_preview, SIGNAL(triggered()), this, SLOT(slotCmdPreview()));

    m_undo = new QAction(QIcon(":/icons/undo.png"), QString::fromUtf8("Zpět"), this);
    m_undo->setShortcut(QKeySequence::Undo);
    m_undo->setEnabled(false);
    connect(m_undo, SIGNAL(triggered()), m_edit, SLOT(undo()));
    connect(m_edit, SIGNAL(undoAvailable(bool)), m_undo, SLOT(setEnabled(bool)));
    m_redo = new QAction(QIcon(":/icons/redo.png"), "Znovu", this);
    m_redo->setShortcut(QKeySequence::Redo);
    m_redo->setEnabled(false);
    connect(m_redo, SIGNAL(triggered()), m_edit, SLOT(redo()));
    connect(m_edit, SIGNAL(redoAvailable(bool)), m_redo, SLOT(setEnabled(bool)));
    m_cut = new QAction(QIcon(":/icons/cut.png"), "Vyjmout", this);
    m_cut->setShortcut(QKeySequence::Cut);
    connect(m_cut, SIGNAL(triggered()), m_edit, SLOT(cut()));
    m_cut->setEnabled(false);
    connect(m_edit, SIGNAL(copyAvailable(bool)), m_cut, SLOT(setEnabled(bool)));
    m_copy = new QAction(QIcon(":/icons/copy.png"), QString::fromUtf8("Kopírovat"), this);
    m_copy->setShortcut(QKeySequence::Copy);
    connect(m_copy, SIGNAL(triggered()), m_edit, SLOT(copy()));
    m_copy->setEnabled(false);
    connect(m_edit, SIGNAL(copyAvailable(bool)), m_copy, SLOT(setEnabled(bool)));
    m_paste = new QAction(QIcon(":/icons/paste.png"), QString::fromUtf8("Vložit"), this);
    m_paste->setShortcut(QKeySequence::Paste);
    m_paste->setEnabled(false);
    connect(m_paste, SIGNAL(triggered()), m_edit, SLOT(paste()));
    m_find = new QAction(QIcon(":/icons/find.png"), QString::fromUtf8("&Najít"), this);
    m_find->setShortcut(QKeySequence::Find);
    m_find->setEnabled(false);
    connect(m_find, SIGNAL(triggered()), this, SLOT(slotCmdFind()));
    m_replace = new QAction(QIcon(":/icons/replace.png"), QString::fromUtf8("Najít a n&ahradit"), this);
    m_replace->setShortcut(QKeySequence::Replace);
    m_replace->setEnabled(false);
    connect(m_replace, SIGNAL(triggered()), this, SLOT(slotCmdFindReplace()));
}

void EditorDialog::createMenu()
{
    QMenu *fileMenu = menuBar()->addMenu("&Soubor");
    fileMenu->addAction(m_open);
    fileMenu->addAction(m_save);
    fileMenu->addSeparator();
    fileMenu->addAction(m_preview);
    fileMenu->addAction(m_print);
    fileMenu->addSeparator();
    QAction *quitAction = new QAction(QIcon(":icons/quit"), QString::fromUtf8("Ukončit"), this);
    quitAction->setShortcut(QKeySequence::Quit);
    quitAction->setMenuRole(QAction::QuitRole);
    connect(quitAction, SIGNAL(triggered()), this, SLOT(close()));
    fileMenu->addAction(quitAction);

    QMenu *editMenu = menuBar()->addMenu(QString::fromUtf8("Úp&ravy"));
    editMenu->addAction(m_undo);
    editMenu->addAction(m_redo);
    editMenu->addSeparator();
    editMenu->addAction(m_cut);
    editMenu->addAction(m_copy);
    editMenu->addAction(m_paste);
    editMenu->addSeparator();
    editMenu->addAction(m_find);
    editMenu->addAction(m_replace);
}

void EditorDialog::createToolbar()
{
    QToolBar *bar = addToolBar("Soubor");
    bar->setStyleSheet("QToolBar { border: 0px }");
    bar->setMovable(false);
    bar->addAction(m_open);
    bar->addAction(m_save);
    bar->addSeparator();
    bar->addAction(m_preview);
    bar->addAction(m_print);
    bar->addSeparator();
    bar->addAction(m_undo);
    bar->addAction(m_redo);
    bar->addSeparator();
    bar->addAction(m_cut);
    bar->addAction(m_copy);
    bar->addAction(m_paste);
    bar->addSeparator();
    bar->addAction(m_find);
    bar->addAction(m_replace);
}

void EditorDialog::setFileName(const QString &filename)
{
    m_filename = filename;
    m_edit->document()->setModified(false);
    m_save->setEnabled(false);
    setTitle();
}

void EditorDialog::setTitle()
{
    QString showName;
    if(m_filename.isEmpty())
    {
        showName = "nepojmenovany.html";
    }
    else
    {
        showName = QFileInfo(m_filename).fileName();
    }
    if(m_edit->document()->isModified())
        setWindowTitle(QString::fromUtf8("Editor šablon - %1 *").arg(showName));
    else
        setWindowTitle(QString::fromUtf8("Editor šablon - %1").arg(showName));
}

void EditorDialog::save()
{
    if(m_filename.isEmpty())
        saveAs();
    else
        saveFile(m_filename);
}

void EditorDialog::saveAs()
{
    QString filename;
#if (QT_VERSION > 0x050000)
    filename = QFileDialog::getSaveFileName(this, "", QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0), QString::fromUtf8("Šablona (*htm *html)"));
#else
    filename = QFileDialog::getSaveFileName(this, "", QDesktopServices::storageLocation(QDesktopServices::HomeLocation), QString::fromUtf8("Šablona (*htm *html)"));
#endif
    if(filename.isEmpty())
        return;
    saveFile(filename);
}

void EditorDialog::saveFile(const QString &filename)
{
    QFile file(filename);
    if (!file.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox::warning(this, QString::fromUtf8("Editor šablon"),
                             QString::fromUtf8("Nemohu uložit soubor %1:\n%2.")
                             .arg(filename)
                             .arg(file.errorString()));
        return;
    }
    QTextStream out(&file);
    out.setCodec("UTF-8");
#ifndef QT_NO_CURSOR
    QApplication::setOverrideCursor(Qt::WaitCursor);
#endif
    out << m_edit->toPlainText();
#ifndef QT_NO_CURSOR
    QApplication::restoreOverrideCursor();
#endif
    setFileName(filename);
}

bool EditorDialog::saveQuestion()
{
    if(m_edit->document()->isModified())
    {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::warning(this, QString::fromUtf8("Editor šablon"),
                    QString::fromUtf8("Soubor byl změněn.\n"
                    "Chcete uložit změny?"),
                    QMessageBox::Save | QMessageBox::No);
        if(ret == QMessageBox::Save)
            return true;
    }
    return false;
}

void EditorDialog::closeEvent(QCloseEvent *event)
{
    if(saveQuestion())
    {
        save();
    }
    event->accept();
}

void EditorDialog::slotDocumentChanged()
{
    if(m_edit->document()->isModified())
        m_save->setEnabled(true);
    else
        m_save->setEnabled(false);
    setTitle();
}

void EditorDialog::slotCmdOpen()
{
    if(saveQuestion())
    {
        save();
    }
    QString filename;
    QStringList items;
    items << QString::fromUtf8("Z výchozí šablony") << "Ze souboru";
    bool ok;
    QString answer = QInputDialog::getItem(this, QString::fromUtf8("Vyberte šablonu"), QString::fromUtf8("Šablona"), items, 0, false, &ok);
    if(ok)
    {
        if(!answer.compare("Ze souboru"))
        {
#if (QT_VERSION > 0x050000)
            filename = QFileDialog::getOpenFileName(this, QString::fromUtf8("Vyberte šablonu"), QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0), QString::fromUtf8("Šablona (*htm *html)"));
#else
            filename = QFileDialog::getOpenFileName(this, QString::fromUtf8("Vyberte šablonu"), QDesktopServices::storageLocation(QDesktopServices::HomeLocation), QString::fromUtf8("Šablona (*htm *html)"));
#endif
        }
        else
        {
            TemplateDialog *dialog = new TemplateDialog(this);
            dialog->setModal(true);
            if(dialog->exec() == QDialog::Accepted)
            {
                filename = QString(":/templates/%1").arg(dialog->templateName());
            }
            else
                return;
        }
        if(!filename.isEmpty())
        {
            QFile file(filename);
            if(!file.open(QFile::ReadOnly | QFile::Text))
            {
                QMessageBox::warning(this, tr("Chyba"),
                    QString::fromUtf8("Nemohu otevřít soubor %1:\n%2.")
                    .arg(filename)
                    .arg(file.errorString()));
                return;
            }
#ifndef QT_NO_CURSOR
            QApplication::setOverrideCursor(Qt::WaitCursor);
#endif
            m_edit->setPlainText(QString::fromUtf8(file.readAll()));
#ifndef QT_NO_CURSOR
            QApplication::restoreOverrideCursor();
#endif
            m_edit->setReadOnly(false);
            m_print->setEnabled(true);
            m_preview->setEnabled(true);
            m_paste->setEnabled(true);
            m_find->setEnabled(true);
            m_replace->setEnabled(true);
            if(filename.startsWith(':'))
                setFileName("");
            else
                setFileName(filename);
        }
    }
}

void EditorDialog::slotCmdSave()
{
    save();
}

void EditorDialog::slotCmdPrint()
{
    QPrinter printer;
    QPrintDialog *dialog = new QPrintDialog(&printer);
    if(dialog->exec() == QDialog::Accepted)
        m_edit->print(&printer);
}

void EditorDialog::slotCmdPreview()
{
    PreviewDialog *dialog = new PreviewDialog(m_edit->toPlainText(), this);
    dialog->setModal(true);
    dialog->show();
}

void EditorDialog::slotCmdFind()
{
    FindreplaceDialog *dialog = new FindreplaceDialog(this, m_edit);
    dialog->setModal(true);
    dialog->show();
}

void EditorDialog::slotCmdFindReplace()
{
    FindreplaceDialog *dialog = new FindreplaceDialog(this, m_edit, true);
    dialog->setModal(true);
    dialog->show();
}

