/***************************************************************
 * Name:      dxfaktura.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include <QApplication>
#ifdef Q_OS_WIN
  #define WIN32_LEAN_AND_MEAN
  #include <windows.h>
#else
  #include <sys/file.h>
  #include <unistd.h>
  #include <stdlib.h>
#endif //Q_OS_WIN
#include <cstdio>
#include <QDesktopServices>
#include "dxfaktura.h"
#include "config.h"
#include "dialogs/aboutdialog.h"
#include "dialogs/firmadialog.h"
#include "dialogs/firmydialog.h"
#include "dialogs/adresardialog.h"
#include "dialogs/printfadialog.h"
#include "dialogs/fakturadialog.h"
#include "dialogs/fatypdialog.h"
#include "dialogs/optionsdialog.h"
#include "dialogs/printseznamfadialog.h"
#include "dialogs/printprijmyvydajedialog.h"
#include "dialogs/editordialog.h"
#include "dxutils.h"
#include "preferences.h"

#ifndef Q_OS_WIN
void dxunlink(void)
{
#if (QT_VERSION > 0x050000)
    unlink(QString(QStandardPaths::standardLocations(QStandardPaths::TempLocation).at(0)+"/dxfaktura-"+getenv("USER")).toLatin1().data());
#else
    unlink(QString(QDesktopServices::storageLocation(QDesktopServices::TempLocation)+"/dxfaktura-"+getenv("USER")).toLatin1().data());
#endif
}
#endif

// For handling Single Instance of Application
#ifdef Q_OS_WIN
bool isInstanceRunningAlready()
#else
bool isInstanceRunningAlready(int argc, char *argv[])
#endif
{
#ifdef Q_OS_WIN
    HANDLE hSingleInstance = NULL;
    hSingleInstance = CreateMutex(NULL, FALSE,TEXT("dxfaktura.exe"));
    DWORD dwLastError = GetLastError();
    if(dwLastError == ERROR_ALREADY_EXISTS)
    {
        MessageBox(NULL, TEXT("dxfaktura už běží"), TEXT("dxfaktura"), MB_OK | MB_ICONINFORMATION);
        CloseHandle(hSingleInstance);
        return true;
    }
#else
#if (QT_VERSION > 0x050000)
    QString path = QStandardPaths::standardLocations(QStandardPaths::TempLocation).at(0)+"/dxfaktura-"+getenv("USER");
#else
    QString path = QDesktopServices::storageLocation(QDesktopServices::TempLocation)+"/dxfaktura-"+getenv("USER");
#endif
    struct flock fl;
    int fd;
    fd = open(path.toLatin1().data(), O_RDWR | O_CREAT, 0600);
    if(fd < 0)
    {
        perror(path.toLatin1().data());
        _exit(1);
    }
    fl.l_start = 0;
    fl.l_len = 0;
    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;
    if(fcntl(fd, F_SETLK, &fl) < 0)
    {
        QApplication a(argc, argv);
        QVBoxLayout *layout = new QVBoxLayout;
        layout->addWidget(new QLabel(QString::fromUtf8("dxfaktura už běží")));
        QPushButton *button = new QPushButton(QString::fromUtf8("&Zavřít"));
        QObject::connect(button, SIGNAL(clicked()), qApp, SLOT(quit()));
        layout->addWidget(button);
        QWidget window;
        window.setLayout(layout);
        window.show();
        a.exec();
        _exit(1);
    }
    atexit(dxunlink);
#endif
    return false;
}

//Constructor
dxfaktura::dxfaktura(bool firstRun, QWidget *parent):
    QMainWindow(parent), m_help(NULL)
{
    resize(preferences.m_w, preferences.m_h);
    move(preferences.m_x, preferences.m_y);
    if(preferences.m_maximized) showMaximized();
    setWindowIcon(QIcon(":/icons/dxfaktura.png"));
    setWindowTitle("dxfaktura");
    QWidget *widget = new QWidget;
    setCentralWidget(widget);
    QVBoxLayout *mainFrame = new QVBoxLayout;
    m_splitter = new QSplitter;
    m_splitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    m_navigator = new QTreeWidget;
    m_navigator->setHeaderHidden(true);
    m_navigator->setRootIsDecorated(false);
    m_navigator->setStyleSheet(QString("QTreeWidget::branch { image: none; }"));
    connect(m_navigator, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(slotNavigation(QTreeWidgetItem*)));
    m_faktury = new Faktury;
    m_faktury->setText(0, "Faktury");
    m_navigator->addTopLevelItem(m_faktury);
    m_vydane = new Vydane(m_faktury);
    m_vydane->setText(0, QString::fromUtf8("Vydané"));
    m_prijate = new Prijate(m_faktury);
    m_prijate->setText(0, QString::fromUtf8("Přijaté"));
    m_navigator->expandAll();
    m_splitter->addWidget(m_navigator);
    m_splitter->addWidget(m_faktury->getPanel());
    mainFrame->addWidget(m_splitter);
    widget->setLayout(mainFrame);
    QList<int> list;
    list << 120 << width()-120;
    m_splitter->setSizes(list);

    createMenu();

    if(firstRun)
    {
        FirmaDialog *dialog = new FirmaDialog(false, 1, this);
        dialog->exec();
        preferences.m_firmaid = 1;
        updatePreferences();
    }

    QSqlQuery q(QString("SELECT nazev FROM firma WHERE id=%1").arg(preferences.m_firmaid));
    q.first();
    m_firma = new QLabel(q.value(0).toString());
    statusBar()->addWidget(m_firma);
}

//Destructor
dxfaktura::~dxfaktura()
{
    preferences.m_x = x();
    preferences.m_y = y();
    preferences.m_w = width();
    preferences.m_h = height();
    preferences.m_maximized = isMaximized();
    preferences.save();
}

void dxfaktura::createMenu()
{
    QMenu *fileMenu = menuBar()->addMenu("&Soubor");
    connect(fileMenu->addAction("&Firma"), SIGNAL(triggered()), this, SLOT(slotCmdFirma()));
    connect(fileMenu->addAction(QString::fromUtf8("&Adresář")), SIGNAL(triggered()), this, SLOT(slotCmdAdresar()));
    fileMenu->addSeparator();
    QMenu *printMenu = fileMenu->addMenu(QIcon(":/icons/print.png"), "&Tisk");
    connect(printMenu->addAction("Seznam faktur"), SIGNAL(triggered()), this, SLOT(slotCmdPrintFaktury()));
    connect(printMenu->addAction(QString::fromUtf8("Příjmy a výdaje")), SIGNAL(triggered()), this, SLOT(slotCmdPrintPrijmyVydaje()));
    fileMenu->addSeparator();
    QAction *quitAction = new QAction(QIcon(":icons/quit"), QString::fromUtf8("Ukončit"), this);
    quitAction->setShortcut(QKeySequence::Quit);
    quitAction->setMenuRole(QAction::QuitRole);
    connect(quitAction, SIGNAL(triggered()), this, SLOT(slotCmdQuit()));
    fileMenu->addAction(quitAction);

    QMenu *fakturaMenu = menuBar()->addMenu("&Faktura");
    QMenu *vydaneMenu = fakturaMenu->addMenu(QString::fromUtf8("Vydané"));
    QAction *vydatFakturuAction = new QAction("Vydat fakturu", this);
#ifdef Q_OS_DARWIN
    vydatFakturuAction->setShortcut(Qt::META+Qt::Key_V);
#else
    vydatFakturuAction->setShortcut(Qt::ALT+Qt::Key_V);
#endif
    connect(vydatFakturuAction, SIGNAL(hovered()), this, SLOT(slotCmdVydane()));
    connect(vydatFakturuAction, SIGNAL(triggered()), this, SLOT(slotCmdVydane()));
    connect(vydatFakturuAction, SIGNAL(triggered()), m_vydane, SLOT(slotCmdAddFaktura()));
    vydaneMenu->addAction(vydatFakturuAction);
    fakturaMenu->addSeparator();
    QMenu *prijateMenu = fakturaMenu->addMenu(QString::fromUtf8("Přijaté"));
    QAction *prijmoutFakturuAction = new QAction(QString::fromUtf8("Přijmout fakturu"), this);
#ifdef Q_OS_DARWIN
    prijmoutFakturuAction->setShortcut(Qt::META+Qt::Key_P);
#else
    prijmoutFakturuAction->setShortcut(Qt::ALT+Qt::Key_P);
#endif
    connect(prijmoutFakturuAction, SIGNAL(hovered()), this, SLOT(slotCmdPrijate()));
    connect(prijmoutFakturuAction, SIGNAL(triggered()), this, SLOT(slotCmdPrijate()));
    connect(prijmoutFakturuAction, SIGNAL(triggered()), m_prijate, SLOT(slotCmdAddFaktura()));
    prijateMenu->addAction(prijmoutFakturuAction);

    QMenu *editMenu = menuBar()->addMenu(QString::fromUtf8("Úp&ravy"));
    QAction *optionsAction = new QAction(QString::fromUtf8("Nastavení"), this);
    optionsAction->setMenuRole(QAction::PreferencesRole);
    connect(optionsAction, SIGNAL(triggered()), this, SLOT(slotCmdOptions()));
    editMenu->addAction(optionsAction);
    editMenu->addSeparator();
    QAction *editorAction = new QAction(QString::fromUtf8("Editor šablon"), this);
    connect(editorAction, SIGNAL(triggered()), this, SLOT(slotCmdEditor()));
    editMenu->addAction(editorAction);

    QMenu *helpMenu = menuBar()->addMenu(QString::fromUtf8("&Nápověda"));
    QAction *helpAction = new QAction(QString::fromUtf8("&Nápověda"), this);
    helpAction->setShortcut(Qt::Key_F1);
    connect(helpAction, SIGNAL(triggered()), this, SLOT(slotCmdHelp()));
    helpMenu->addAction(helpAction);
    helpMenu->addSeparator();
    QAction *aboutAction = new QAction(QIcon(":/icons/dxfaktura.png"), "&O programu", this);
    aboutAction->setMenuRole(QAction::AboutRole);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(slotCmdAbout()));
    helpMenu->addAction(aboutAction);
    QAction *aboutQtAction = new QAction(QPixmap(":/trolltech/qmessagebox/images/qtlogo-64.png"), "O &Qt", this);
    aboutQtAction->setMenuRole(QAction::AboutQtRole);
    connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
    helpMenu->addAction(aboutQtAction);
}

void dxfaktura::slotCmdQuit()
{
    qApp->quit();
}

void dxfaktura::slotCmdHelp()
{
    if(!m_help)
    {
        m_help = new HelpDialog(this);
        m_help->setModal(true);
    }
    m_help->show();
    m_help->raise();
    m_help->activateWindow();
}

void dxfaktura::slotCmdAbout()
{
    AboutDialog *about = new AboutDialog(this);
    about->setModal(true);
    about->show();
}

void dxfaktura::slotCmdFirma()
{
    FirmyDialog *dialog = new FirmyDialog(this);
    dialog->setModal(true);
    dialog->exec();
    preferences.m_firmaid = dialog->selectedID();
    QSqlQuery q(QString("SELECT nazev FROM firma WHERE id=%1").arg(preferences.m_firmaid));
    q.first();
    m_firma->setText(q.value(0).toString());
    updatePreferences();
    m_faktury->firmaChanged();
    m_prijate->firmaChanged();
    m_vydane->firmaChanged();
}

void dxfaktura::slotCmdAdresar()
{
    AdresarDialog *dialog = new AdresarDialog(false, this);
    dialog->setModal(true);
    dialog->exec();
    m_faktury->firmaChanged();
    m_prijate->firmaChanged();
    m_vydane->firmaChanged();
}

void dxfaktura::slotCmdVydane()
{
    if(m_navigator->currentItem() != m_vydane) m_navigator->setCurrentItem(m_vydane);
}

void dxfaktura::slotCmdPrijate()
{
    if(m_navigator->currentItem() != m_prijate) m_navigator->setCurrentItem(m_prijate);
}

void dxfaktura::slotNavigation(QTreeWidgetItem *current)
{
    if(current)
    {
        QWidget *old = m_splitter->widget(1);
        old->setParent(0);
        dxitem *item = static_cast<dxitem*>(current);
        QWidget *widget = item->getPanel();
        m_splitter->insertWidget(1, widget);
        widget->show();
    }
}

void dxfaktura::slotCmdOptions()
{
    OptionsDialog *dialog = new OptionsDialog(this);
    dialog->setModal(true);
    dialog->exec();
    m_faktury->firmaChanged();
    m_prijate->firmaChanged();
    m_vydane->firmaChanged();
}

void dxfaktura::slotCmdPrintFaktury()
{
    PrintSeznamFaDialog *dialog = new PrintSeznamFaDialog(this);
    dialog->setModal(true);
    dialog->show();
}

void dxfaktura::slotCmdPrintPrijmyVydaje()
{
    PrintPrijmyVydajeDialog *dialog = new PrintPrijmyVydajeDialog(this);
    dialog->setModal(true);
    dialog->show();
}

void dxfaktura::slotCmdEditor()
{
    EditorDialog *dialog = new EditorDialog(this);
    dialog->show();
}

void dxfaktura::updatePreferences()
{
    QSqlQuery q(QString("SELECT rok,vydana,prijata,splatnost,"
                        "faprijataneplatce,faprijataplatce,favydananeplatce,favydanaplatce,zalohaprijataneplatce,"
                        "zalohaprijataplatce,zalohavydananeplatce,zalohavydanaplatce"
                        " FROM settings WHERE firmaid=%1").arg(preferences.m_firmaid));
    q.first();
    preferences.m_rok = q.value(0).toInt();
    preferences.m_prefixVydana = q.value(1).toString();
    preferences.m_prefixPrijata = q.value(2).toString();
    preferences.m_splatnost = q.value(3).toInt();
    preferences.m_faprijataneplatce = preferences.validTemplate(q.value(4).toString());
    preferences.m_faprijataplatce = preferences.validTemplate(q.value(5).toString());
    preferences.m_favydananeplatce = preferences.validTemplate(q.value(6).toString());
    preferences.m_favydanaplatce = preferences.validTemplate(q.value(7).toString());
    preferences.m_zalohaprijataneplatce = preferences.validTemplate(q.value(8).toString());
    preferences.m_zalohaprijataplatce = preferences.validTemplate(q.value(9).toString());
    preferences.m_zalohavydananeplatce = preferences.validTemplate(q.value(10).toString());
    preferences.m_zalohavydanaplatce = preferences.validTemplate(q.value(11).toString());
    q.exec(QString("SELECT server,port,ssl,autentifikace,username,password,sender,subject,text FROM smtp WHERE firmaid=%1").arg(preferences.m_firmaid));
    q.first();
    preferences.m_server = q.value(0).toString();
    preferences.m_port = q.value(1).toInt();
    preferences.m_ssl = q.value(2).toBool();
    preferences.m_autentifikace = q.value(3).toBool();
    preferences.m_username = q.value(4).toString();
    preferences.m_password = q.value(5).toString();
    preferences.m_sender = q.value(6).toString();
    preferences.m_subject = q.value(7).toString();
    preferences.m_text = q.value(8).toString();
}

//nacteni Qt prekladu
void loadTranslation()
{
    QString langPath = "";
    QStringList paths;
    QString appdir = QCoreApplication::applicationDirPath();
    paths.append(appdir);
    paths.append(appdir + "/../share/" + QCoreApplication::applicationName().toLower());
    paths.append(appdir + "/../Resources");
    foreach(const QString& path, paths)
    {
        if(QFile::exists(path + "/translations/"))
        {
            langPath = path + "/translations/";
            break;
        }
    }
    static QTranslator qt_translator;
    if(!langPath.isEmpty())
    {
        qt_translator.load("qt_cs", langPath);
    }
    else
    {
        qt_translator.load("qt_cs", QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    }
    QCoreApplication::installTranslator(&qt_translator);
}

const QString USAGE_MSG = QString::fromUtf8("\
\nPoužití: dxfaktura [volby] \n\
\n\
    [volby] můžou být následující:\n\
\n\
        -h, --help         Zobrazí (tuto) nápovědu a skončí.\n\
        -v, --version      Zobrazí informaci o verzi a skončí.\n\
\n");

int main(int argc, char *argv[])
{
#ifdef Q_OS_WIN
    QCoreApplication::addLibraryPath(".");
#endif
    for(int i=0; i<argc; ++i)
    {
        if(QString::compare(argv[i],"-v")==0 || QString::compare(argv[i],"--version")==0)
        {
            fprintf(stdout, "dxfaktura %s\n", VERSION);
            exit(0);
        }
        else if(QString::compare(argv[i],"-h")==0 || QString::compare(argv[i],"--help")==0)
        {
            fprintf(stdout, "%s\n", USAGE_MSG.toUtf8().constData());
            exit(0);
        }
        else if(i!=0)
        {
            fprintf(stdout, "Neznámá volba: %s\n", argv[i]);
            exit(0);
        }
    }
#ifdef Q_OS_WIN
    if(isInstanceRunningAlready())
        return 0;
#else
    if(isInstanceRunningAlready(argc, argv))
        return 0;
#endif

    QString path;
#ifdef Q_OS_WIN
    path = QProcessEnvironment::systemEnvironment().value("APPDATA")+"/dxfaktura";
#else
#if (QT_VERSION > 0x050000)
    path = QStandardPaths::standardLocations(QStandardPaths::HomeLocation).at(0)+"/.dxfaktura";
#else
    path = QDesktopServices::storageLocation(QDesktopServices::HomeLocation)+"/.dxfaktura";
#endif
#endif
    QDir dir;
    if(!dir.exists(path)) dir.mkpath(path);
    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
    db.setDatabaseName(path+"/faktura.db");
    db.open();
    QSqlQuery query;
    query.exec("PRAGMA foreign_keys = ON;");
    bool firstRun = db.tables().contains("firma") ? false : true;
    if(firstRun) dxutils::createDB();

    preferences.load();
    QLocale::setDefault(QLocale(QLocale::Czech, QLocale::CzechRepublic));
    QApplication a(argc, argv);
    QApplication::setOrganizationName("dxSolutions");
    QApplication::setApplicationName("dxfaktura");
    QApplication::setApplicationVersion(VERSION);
#ifdef Q_OS_DARWIN
    QCoreApplication::setAttribute(Qt::AA_DontShowIconsInMenus, true);
#endif
    loadTranslation();
    dxfaktura w(firstRun);
    w.show();
    return a.exec();
}
