/***************************************************************
 * Name:      faparser.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2013
 * License:   GPL3
 **************************************************************/

#include "faparser.h"
#include "dxutils.h"
#include "preferences.h"

const QString fakturaTags[] = {"{faktura-cislo}","{faktura-celkem}","{faktura-uhrada}","{faktura-banka}",
                               "{faktura-vystaveni}","{faktura-splatnost}","{faktura-duzp}",
                               "{faktura-ks}","{faktura-vs}","{faktura-mena}"};

const int numFakturaTags = (sizeof(fakturaTags)/sizeof(fakturaTags[0]));

bool isRightFakturaTag(const QString &line)
{
    for(int i=0; i<numFakturaTags; i++)
    {
        if(line.contains(fakturaTags[i]))
            return true;
    }
    return false;
}

const QString firmaTags[] = {"{firma-nazev}","{firma-ulice}","{firma-registrace}","{firma-logo}","{firma-razitko}",
                             "{firma-psc}","{firma-misto}","{firma-zeme}","{firma-ic}","{firma-dic}",
                             "{firma-telefon}","{firma-email}","{firma-www}"};

const int numFirmaTags = (sizeof(firmaTags)/sizeof(firmaTags[0]));

bool isRightFirmaTag(const QString &line)
{
    for(int i=0; i<numFirmaTags; i++)
    {
        if(line.contains(firmaTags[i]))
            return true;
    }
    return false;
}

const QString adresarTags[] = {"{adresar-nazev}","{adresar-ulice}","{adresar-psc}","{adresar-misto}",
                             "{adresar-zeme}","{adresar-ic}","{adresar-dic}"};

const int numAdresarTags = (sizeof(adresarTags)/sizeof(adresarTags[0]));

bool isRightAdresarTag(const QString &line)
{
    for(int i=0; i<numAdresarTags; i++)
    {
        if(line.contains(adresarTags[i]))
            return true;
    }
    return false;
}

const QString polozkaTags[] = {"{polozka-popis}","{polozka-pocet}","{polozka-cenamj}",
                             "{polozka-sazba}","{polozka-dph}","{polozka-celkem}"};

const int numPolozkaTags = (sizeof(polozkaTags)/sizeof(polozkaTags[0]));

bool isRightPolozkaTag(const QString &line)
{
    for(int i=0; i<numPolozkaTags; i++)
    {
        if(line.contains(polozkaTags[i]))
            return true;
    }
    return false;
}

const QString sazbaTags[] = {"{dph-sazba}","{dph-zaklad}","{dph-dph}"};

const int numSazbaTags = (sizeof(sazbaTags)/sizeof(sazbaTags[0]));

bool isRightSazbaTag(const QString &line)
{
    for(int i=0; i<numSazbaTags; i++)
    {
        if(line.contains(sazbaTags[i]))
            return true;
    }
    return false;
}

FaParser::FaParser(int id) :
    m_id(id)
{
    QSqlQuery faktura(QString("SELECT platce FROM faktura WHERE id = %1").arg(m_id));
    faktura.first();
    m_platce = faktura.value(0).toBool();
}

QString FaParser::parseFile(const QString &filename)
{
    QString line, text;
    QFile file(filename);
    if(!file.open(QFile::ReadOnly | QFile::Text))
    {
        return QString::fromUtf8("<html><head></head><body>Nemohu otevřít šablonu: %1</body></html>").arg(filename);
    }
    while(!file.atEnd())
    {
        line = QString::fromUtf8(file.readLine());
        if(isRightFakturaTag(line))
        {
            line = parseFakturaLine(line);
        }
        if(isRightFirmaTag(line))
        {
            line = parseFirmaLine(line);
        }
        if(isRightAdresarTag(line))
        {
            line = parseAdresarLine(line);
        }
        if(isRightPolozkaTag(line))
        {
            line = parsePolozkaLine(line);
        }
        if(m_platce && isRightSazbaTag(line))
        {
            line = parseSazbaLine(line);
        }
        text += line;
    }
    return text;
}

QString FaParser::parseFakturaLine(QString line)
{
    QSqlQuery faktura(QString("SELECT * FROM faktura WHERE id = %1").arg(m_id));
    faktura.first();
    if(line.contains("{faktura-cislo}"))
    {
        line = line.replace("{faktura-cislo}",faktura.value(faktura.record().indexOf("cislo")).toString());
    }
    if(line.contains("{faktura-celkem}"))
    {
        line = line.replace("{faktura-celkem}",QString("%L1").arg(faktura.value(faktura.record().indexOf("celkem")).toFloat(), 0,'f',2));
    }
    if(line.contains("{faktura-vystaveni}"))
    {
        line = line.replace("{faktura-vystaveni}",faktura.value(faktura.record().indexOf("vystaveni")).toString());
    }
    if(line.contains("{faktura-splatnost}"))
    {
        line = line.replace("{faktura-splatnost}",faktura.value(faktura.record().indexOf("splatnost")).toString());
    }
    if(line.contains("{faktura-duzp}"))
    {
        line = line.replace("{faktura-duzp}",faktura.value(faktura.record().indexOf("duzp")).toString());
    }
    if(line.contains("{faktura-uhrada}"))
    {
        int id = faktura.value(faktura.record().indexOf("uhradaid")).toInt();
        QSqlQuery uhrada(QString("SELECT uhrada FROM fauhrada WHERE id = %1").arg(id));
        uhrada.first();
        line = line.replace("{faktura-uhrada}",uhrada.value(0).toString());
    }
    if(line.contains("{faktura-mena}"))
    {
        int id = faktura.value(faktura.record().indexOf("menaid")).toInt();
        QSqlQuery mena(QString("SELECT zkratka FROM mena WHERE id = %1").arg(id));
        mena.first();
        line = line.replace("{faktura-mena}",mena.value(0).toString());
    }
    if(line.contains("{faktura-banka}"))
    {
        int id = faktura.value(faktura.record().indexOf("uhradaid")).toInt();
        QSqlQuery uhrada(QString("SELECT hotove FROM fauhrada WHERE id = %1").arg(id));
        uhrada.first();
        if(uhrada.value(0).toBool()) //hotove
        {
            line = "";
        }
        else
        {
            int id = faktura.value(faktura.record().indexOf("bankaid")).toInt();
            QSqlQuery banka(QString("SELECT cislo,banka FROM banka WHERE id = %1").arg(id));
            banka.first();
            line.replace("{faktura-banka}",QString("%1/%2").arg(banka.value(0).toString()).arg(banka.value(1).toString()));
        }
    }
    if(line.contains("{faktura-vs}"))
    {
        int id = faktura.value(faktura.record().indexOf("uhradaid")).toInt();
        QSqlQuery uhrada(QString("SELECT hotove FROM fauhrada WHERE id = %1").arg(id));
        uhrada.first();
        if(uhrada.value(0).toBool()) //hotove
        {
            line = "";
        }
        else
        {
            line.replace("{faktura-vs}",faktura.value(faktura.record().indexOf("vs")).toString());
        }
    }
    if(line.contains("{faktura-ks}"))
    {
        int id = faktura.value(faktura.record().indexOf("uhradaid")).toInt();
        QSqlQuery uhrada(QString("SELECT hotove FROM fauhrada WHERE id = %1").arg(id));
        uhrada.first();
        if(uhrada.value(0).toBool()) //hotove
        {
            line = "";
        }
        else
        {
            line.replace("{faktura-ks}",faktura.value(faktura.record().indexOf("ks")).toString());
        }
    }
    return line;
}

QString FaParser::parseFirmaLine(QString line)
{
    QSqlQuery faktura(QString("SELECT firmaid FROM faktura WHERE id = %1").arg(m_id));
    faktura.first();
    int id = faktura.value(0).toInt();
    QSqlQuery firma(QString("SELECT * FROM firma_view WHERE id = %1").arg(id));
    firma.first();
    if(line.contains("{firma-nazev}"))
    {
        line.replace("{firma-nazev}",firma.value(firma.record().indexOf("nazev")).toString());
    }
    if(line.contains("{firma-ulice}"))
    {
        line.replace("{firma-ulice}",QString("%1 %2").arg(firma.value(firma.record().indexOf("ulice")).toString())
                            .arg(firma.value(firma.record().indexOf("cp")).toString()));
    }
    if(line.contains("{firma-misto}"))
    {
        line.replace("{firma-misto}",firma.value(firma.record().indexOf("misto")).toString());
    }
    if(line.contains("{firma-psc}"))
    {
        line.replace("{firma-psc}",firma.value(firma.record().indexOf("psc")).toString());
    }
    if(line.contains("{firma-zeme}"))
    {
        line.replace("{firma-zeme}",firma.value(firma.record().indexOf("zeme")).toString());
    }
    if(line.contains("{firma-registrace}"))
    {
        line.replace("{firma-registrace}",firma.value(firma.record().indexOf("registrace")).toString());
    }
    if(line.contains("{firma-ic}"))
    {
        line.replace("{firma-ic}",firma.value(firma.record().indexOf("ic")).toString());
    }
    if(line.contains("{firma-dic}"))
    {
        line.replace("{firma-dic}",firma.value(firma.record().indexOf("dic")).toString());
    }
    if(line.contains("{firma-telefon}"))
    {
        line.replace("{firma-telefon}",firma.value(firma.record().indexOf("telefon")).toString());
    }
    if(line.contains("{firma-email}"))
    {
        line.replace("{firma-email}",firma.value(firma.record().indexOf("email")).toString());
    }
    if(line.contains("{firma-www}"))
    {
        line.replace("{firma-www}",firma.value(firma.record().indexOf("www")).toString());
    }
    if(line.contains("{firma-logo}"))
    {
        QString logo = firma.value(firma.record().indexOf("logo")).toString();
        if(logo.isEmpty()) line.replace("{firma-logo}","");
        else line.replace("{firma-logo}",QString("<br><img src=\"file:////%1\" />").arg(logo));
    }
    if(line.contains("{firma-razitko}"))
    {
        QString razitko = firma.value(firma.record().indexOf("razitko")).toString();
        if(razitko.isEmpty()) line.replace("{firma-razitko}","<br><br><br><br><br><br><br><br><br>");
        else line.replace("{firma-razitko}",QString("<br><img src=\"file:////%1\" /><br>").arg(razitko));
    }
    return line;
}

QString FaParser::parseAdresarLine(QString line)
{
    QSqlQuery faktura(QString("SELECT adresarid FROM faktura WHERE id = %1").arg(m_id));
    faktura.first();
    int id = faktura.value(0).toInt();
    QSqlQuery adresar(QString("SELECT * FROM adresar_view WHERE id = %1").arg(id));
    adresar.first();
    if(line.contains("{adresar-nazev}"))
    {
        line.replace("{adresar-nazev}",adresar.value(adresar.record().indexOf("nazev")).toString());
    }
    if(line.contains("{adresar-ulice}"))
    {
        line.replace("{adresar-ulice}",QString("%1 %2").arg(adresar.value(adresar.record().indexOf("ulice")).toString())
                            .arg(adresar.value(adresar.record().indexOf("cp")).toString()));
    }
    if(line.contains("{adresar-misto}"))
    {
        line.replace("{adresar-misto}",adresar.value(adresar.record().indexOf("misto")).toString());
    }
    if(line.contains("{adresar-psc}"))
    {
        line.replace("{adresar-psc}",adresar.value(adresar.record().indexOf("psc")).toString());
    }
    if(line.contains("{adresar-zeme}"))
    {
        line.replace("{adresar-zeme}",adresar.value(adresar.record().indexOf("zeme")).toString());
    }
    if(line.contains("{adresar-ic}"))
    {
        line.replace("{adresar-ic}",adresar.value(adresar.record().indexOf("ic")).toString());
    }
    if(line.contains("{adresar-dic}"))
    {
        line.replace("{adresar-dic}",adresar.value(adresar.record().indexOf("dic")).toString());
    }
    return line;
}

QString FaParser::parsePolozkaLine(QString line)
{
    if(line.contains("{polozka-popis}"))
    {
        QString text;
        QSqlQuery polozka;
        polozka.exec(QString("SELECT popis FROM polozka WHERE fakturaid = %1").arg(m_id));
        while(polozka.next())
        {
            text += QString("%1<br>").arg(polozka.value(0).toString());
        }
        line.replace("{polozka-popis}",text);
    }
    if(line.contains("{polozka-pocet}"))
    {
        QString text;
        QSqlQuery polozka;
        polozka.exec(QString("SELECT pocet,mj FROM polozka WHERE fakturaid = %1").arg(m_id));
        while(polozka.next())
        {
            text += QString("%L1 %2<br>").arg(polozka.value(0).toFloat(), 0,'f',2).arg(polozka.value(1).toString());
        }
        line.replace("{polozka-pocet}",text);
    }
    if(line.contains("{polozka-cenamj}"))
    {
        QString text;
        QSqlQuery polozka;
        polozka.exec(QString("SELECT id,cenamj,bezdph FROM polozka WHERE fakturaid = %1").arg(m_id));
        while(polozka.next())
        {
            if(polozka.value(2).toBool()) text += QString("%L1<br>").arg(polozka.value(1).toFloat(), 0,'f',2);
            else
            {
                QSqlQuery sazba;
                sazba.exec(QString("SELECT sazba FROM dphsazby INNER JOIN polozka ON dphsazby.id=sazbaid WHERE polozka.id = %1").arg(polozka.value(0).toInt()));
                sazba.first();
                text += QString("%L1<br>").arg(polozka.value(1).toFloat()*(1-ceil4(sazba.value(0).toDouble()/(100.0+sazba.value(0).toDouble()))), 0,'f',2);
            }
        }
        line.replace("{polozka-cenamj}",text);
    }
    if(line.contains("{polozka-sazba}"))
    {
        QString text;
        QSqlQuery polozka;
        polozka.exec(QString("SELECT sazba FROM dphsazby INNER JOIN polozka ON dphsazby.id=sazbaid WHERE fakturaid = %1").arg(m_id));
        while(polozka.next())
        {
            text += QString("%1<br>").arg(polozka.value(0).toInt());
        }
        line.replace("{polozka-sazba}",text);
    }
    if(line.contains("{polozka-dph}"))
    {
        QString text;
        QSqlQuery polozka;
        polozka.exec(QString("SELECT dph FROM polozka WHERE fakturaid = %1").arg(m_id));
        while(polozka.next())
        {
            text += QString("%L1<br>").arg(polozka.value(0).toFloat(), 0,'f',2);
        }
        line.replace("{polozka-dph}",text);
    }
    if(line.contains("{polozka-celkem}"))
    {
        QString text;
        QSqlQuery polozka;
        polozka.exec(QString("SELECT celkem FROM polozka WHERE fakturaid = %1").arg(m_id));
        while(polozka.next())
        {
            text += QString("%L1<br>").arg(polozka.value(0).toFloat(), 0,'f',2);
        }
        line.replace("{polozka-celkem}",text);
    }
    return line;
}

QString FaParser::parseSazbaLine(QString line)
{
    QString sazba, zaklad, dph;
    QSqlQuery q;
    q.exec(QString("SELECT sum(celkem-dph), sum(dph), sazba "
                    "FROM polozka INNER JOIN dphsazby ON sazbaid=dphsazby.id WHERE fakturaid=%1 GROUP BY sazbaid ORDER BY sazba DESC").arg(m_id));
    while(q.next())
    {
        zaklad += QString("%L1<br>").arg(q.value(0).toFloat(),0,'f',2);
        dph += QString("%L1<br>").arg(q.value(1).toFloat(),0,'f',2);
        sazba += QString("%1%<br>").arg(q.value(2).toInt());
    }
    line.replace("{dph-zaklad}", zaklad);
    line.replace("{dph-dph}", dph);
    line.replace("{dph-sazba}", sazba);
    return line;
}
